--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local addonName, NS = ...

local Lib = NS.FizzleLib

local function MenuOnEnter(self)
	self:SetScript("OnUpdate", nil)
	self.TimeToLeave = nil
end

local function MenuOnLeave(self)
	local parent = self:GetParent()
	if not (self:IsMouseOver() and parent:IsMouseOver()) then  -- GetMouseFocus() maybe??? -- and parent.DropButton:IsMouseOver()) then -- over the button should be over the droplist control
		self.TimeToLeave = 0.3
		self:SetScript("OnUpdate", function(self, elapsed)
			self.TimeToLeave = self.TimeToLeave - elapsed
			if self.TimeToLeave <= 0 then
				self:SetScript("OnUpdate", nil)
				self:Hide()
			end
		end)
	end
end

local function DropMenu_Init(self, override)
	local text
	if not self.Menu then
		self = self:GetParent() -- Init starts at parent, not the drop menu
	end
	self.Key = nil
	local initval = self.Data[self.Index]
	for i=1, #self.Menu.Table do
		local row = self.Menu.Table[i]
		if row[self.Menu.Value] == initval then
			text = row[self.Menu.Display]
			self.Key = i
			break
		end
	end
	if not text then
		if override then
			text = initval
			if not text then
				text = ""
			end
		else
			text = ""
		end
	end
	self.SelectedText:SetText(text)
end

local function PositionMenu(self)
	if self:GetBottom() < UIParent:GetBottom() then
		self:ClearAllPoints()
		self:SetPoint("BOTTOMLEFT", self:GetParent(), "TOPLEFT")
	else
		self:ClearAllPoints()
		self:SetPoint("TOPLEFT", self:GetParent(), "BOTTOMLEFT")
	end
	self:Show()
end

local function DropMenuShow(self)
	PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
	if not self.DefaultPadding then
		local DefaultPadding
		local t = { self.Buttons[1]:GetPoint(1) }
		for i=1, #t do
			if type(t[i]) == "number" then
				DefaultPadding = t[i]
				break
			end
		end
		if DefaultPadding then
			self.DefaultPadding = DefaultPadding
		end
		DefaultPadding = nil
		t = { self.Buttons[1].Text:GetPoint(1) }
		for i=1, #t do
			if type(t[i]) == "number" then
				DefaultPadding = t[i]
				break
			end
		end
		if DefaultPadding then
			self.DefaultPadding = self.DefaultPadding and self.DefaultPadding + DefaultPadding or DefaultPadding
		end
		if not self.DefaultPadding then
			self.DefaultPadding = 2
		end
	end
	if self:IsShown() then
		self:Hide()
		return
	end
	local parent = self:GetParent()
	if self.ScrollList then
		self:SetSize(self.ScrollList:GetWidth()+32, self.ScrollList:GetHeight()+10)
		local scrollwidth = (self.ScrollList.ScrollBar:IsShown() and self.ScrollList.ScrollBar:GetWidth()) or 0
		self:SetSize(self.ScrollList:GetWidth() + (scrollwidth * 2), self.ScrollList.ListHeight + 10)
		PositionMenu(self)
		return
	end
	local numButtons = #self.Buttons
	local widest = 0
	local count = 0
	for i=1, numButtons do
		if not parent.Excludes or not parent.Excludes[self.Table[i][self.Value]] then 
			count = count + 1
			local button = self.Buttons[count]
			button.Text:SetText(self.Table[i][self.Display])
			button.Value = self.Table[i][self.Value]
			button.Desc = self.Table[i][self.Desc]
			button.index = i
			button:Show()
			local width = button.Text:GetWidth()
			if width > widest then
				widest = width
			end
		end
	end
	for i=1, numButtons do
		if i > count then
			self.Buttons[i]:Hide()
		else
--			self.Buttons[i]:SetWidth(widest)
		end
	end
--	self:SetSize((widest + (self.DefaultPadding * 2)) + self:GetParent().Padding, (count*20)+10)
	self:SetSize(self:GetParent():GetWidth(), (count*20)+10)
	PositionMenu(self)
end

local function RunHiders(self, show)
	local hiders = self:GetParent().Hiders
	if not hiders then return end
	for i=1, #hiders do
		hiders[i]:SetShown(show)
	end
end

local function SetButtonSize(self)
	local size = self:GetHeight()
	self.DropButton:SetSize(size-4, size-4)
	if self.Menu and self.Menu.ScrollList then
		self.Menu.ScrollList:SetWidth(self:GetWidth() - self.DropButton:GetWidth() * 2)
	end
end

local function CreateDropHandler(name, parent, table) --, data, index)--, desc --, maxdrops) -- see below
	local f = Lib:CreateDropBackdrop(name, parent, "Button")
	f:SetSize(100, 30)
	f.Table = table
	f:SetBackdropColor(0, 0, 0, 0.4)
	f.DropButton = CreateFrame("Button", "$parentButton", f)
	SetButtonSize(f)
	f.Padding = 0
	f.DropButton:SetPoint("RIGHT", -2, 0)

	f.DropButton:SetNormalTexture(Lib.TexturePath.."DropButtonNormal")
	
	f.DropButton:SetPushedTexture(Lib.TexturePath.."DropButtonPushed")
	
	f.DropButton:SetDisabledTexture(Lib.TexturePath.."DropButtonDisabled")

	f.DropButton:SetHighlightTexture(Lib.TexturePath.."DropButtonHighlight", "ADD")
	f.DropButton:SetScript("OnClick", function(self)
		DropMenuShow(self:GetParent().Menu)
	end)
	f.DropButton:SetScript("OnEnter", function(self)
		MenuOnEnter(self:GetParent().Menu)
	end)
	f.DropButton:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
--	f.SelectedText = Lib:CreateFont("Setting", f, "", "LEFT", "CENTER")
	f.SelectedText = Lib:CreateFont("Setting", f, "", "LEFT", "MIDDLE")
	f.SelectedText:SetPoint("LEFT", f, 5, 0)
	f:SetScript("OnSizeChanged", function(self)
			SetButtonSize(self)
		end)
	f:SetScript("OnEnter", function(self)
		MenuOnEnter(self.Menu)
	end)
	f:SetScript("OnLeave", function(self)
		MenuOnLeave(self.Menu)
	end)
	f:HookScript("OnEnable", function(self)
		self.DropButton:SetEnabled(true)
		self.Menu:SetEnabled(true)
	end)
	f:HookScript("OnDisable", function(self)
		self.DropButton:SetEnabled(false)
		self.Menu:SetEnabled(false)
	end)

	function f:AddDropHider(frame)
		if not self.Hiders then
			self.Hiders = {}
		end
		tinsert(self.Hiders, frame)
	end
	function f:AddPadding(padding)
		self.Padding = padding or 0
	end
	function f:ExcludeValue(value)
		if not self.Excludes then
			self.Excludes = {}
			self.NumExludes = 0
		end
		if not self.Excludes[value] then -- no duplicate excudes
			self.Excludes[value]=true
			self.NumExludes = self.NumExludes + 1
		end
	end
	Lib:AddLabelFuncs(f)
	return f
end

local function CreateMenuHandler(self, data, index, table, display, value, maxdrops)
	local f = Lib:CreateDropBackdrop("Menu", self, "Button")
	f:SetSize(#table, #table) -- we will be adjusting the size when we know more -- #table can be temporary until scroll code done
	f:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	f:SetBackdropColor(0, 0, 0, 1)
	f.LeaveTimer = 0.5
	f:Hide()
	f.Table = table
	f.Display = display
	f.Value = value
	f.Index = index
	f:SetScript("OnShow", function(self)
		RunHiders(self)
	end)
	f:SetScript("OnEnter", function(self)
			MenuOnEnter(self)
	end)
	f:SetScript("OnLeave", function(self)
			MenuOnLeave(self)
	end)
	f:SetScript("OnHide", function(self)
		self:SetScript("OnUpdate", nil)
		self.TimeToLeave = nil
		RunHiders(self, true)
	end)
	function f:SetLeaveTimer(timer)
		f.LeaveTimer = timer
	end
	if maxdrops then
		f.ScrollList = Lib:CreateScrollList("SrollList", f, data, index, table, display, value, maxdrops) -- ,buttonupdatedisplay, buttonlayout)--Default buttons etc. for now
		f.Buttons = f.ScrollList.Buttons
		f.ScrollList:SetSize(100, 120) -- Temp
		f.ScrollList:SetPoint("TOPLEFT", 5, -5)
		f.ScrollList:HookScript("OnEnter", function(self)
			MenuOnEnter(self:GetParent())
		end)
		f.ScrollList:HookScript("OnLeave", function(self)
			MenuOnLeave(self:GetParent())
		end)
		local ScrollOpts = { FauxScrollFrame_GetChildFrames(f.ScrollList) }
		tremove(ScrollOpts, 2)
		for k, v in pairs(ScrollOpts) do
			v:HookScript("OnEnter", function(self)
				MenuOnEnter(f)
			end)
			v:HookScript("OnLeave", function(self)
				MenuOnEnter(f)
			end)
		end
		for i=1, #f.ScrollList.Buttons do
			f.ScrollList.Buttons[i]:HookScript("OnClick", function(self)
				self:GetParent():GetParent():GetParent().SelectedText:SetText(self:GetParent().Table[self.index][self:GetParent().Display])
				self:GetParent():GetParent():Hide()
			end)
			f.ScrollList.Buttons[i]:HookScript("OnEnter", function(self)
				MenuOnEnter(self:GetParent():GetParent())
				local parent = self:GetParent()
			end)
			f.ScrollList.Buttons[i]:HookScript("OnLeave", function(self)
				MenuOnLeave(self:GetParent():GetParent())
			end)
		end
	else
		f.Buttons = {}
		for i=1, #f.Table do
			local button = CreateFrame("CheckButton", "$parentButton"..i, f)
			f.Buttons[i] = button
			button:SetSize(20, 20)
			Lib.CreateDefaultListButton(button)
			button.id = i
			button:HookScript("OnClick", function(self)
				local menu = self:GetParent()
				local parent = menu:GetParent()
				local text = self.Text:GetText()
				parent.Key = nil
				for k, v in pairs(menu.Table) do
					if v[menu.Display] == text then
						parent.SelectedText:SetText(v[menu.Display])
						parent.Data[parent.Index] = v[menu.Value]
						parent.Key = k
						break
					end
				end
				menu:Hide()
				if parent.PostUpdate then
					parent:PostUpdate()--, parent.Data, parent.Index)
				end
			end)
			button:HookScript("OnEnter", function(self)
				MenuOnEnter(self:GetParent())
				local parent = self:GetParent()
				if not parent.Table[self.index].tooltip then return end
				GameTooltip:SetOwner(self, anchor or "ANCHOR_LEFT")
				GameTooltip:SetText(parent.Table[self.index].tooltip, nil, nil, nil, nil, true)
			end)
			button:HookScript("OnLeave", function(self)
				MenuOnLeave(self:GetParent())
				GameTooltip:Hide()
			end)

			if i == 1 then
				button:SetPoint("TOPLEFT", f, 3, -5)
				button:SetPoint("TOPRIGHT", f)
			else
				button:SetPoint("TOPLEFT", f.Buttons[i-1], "BOTTOMLEFT")
				button:SetPoint("TOPRIGHT", f.Buttons[i-1], "BOTTOMRIGHT")
			end
		end
	end
	return f
end

function Lib:CreateDropList(name, parent, data, index, table, display, value, maxdrops)
	local f = CreateDropHandler(name, parent, table)--, data, index)--, table, display, value) -- , maxdrops, desc
	f.Menu = CreateMenuHandler(f, data, index, table, display, value, maxdrops) --, display, value, desc)
	Lib.InitData(f, data, index, DropMenu_Init)
	return f
end
